float4x4 World;
float4x4 View;
float4x4 Projection;
float3 LightPosition;
float3 CameraPosition;
float scale2;			//radius of planet squared

texture ringtex;
sampler RingSampler = sampler_state 
{
    texture = <ringtex>;
    magfilter	= LINEAR; 
	minfilter	= LINEAR; 
	mipfilter	= LINEAR; 
	AddressU	= CLAMP;  
	AddressV	= CLAMP;
};

texture noisetex;
sampler NoiseSampler = sampler_state 
{
    texture = <noisetex>;
    magfilter	= LINEAR; 
	minfilter	= LINEAR; 
	mipfilter	= LINEAR; 
	AddressU	= CLAMP;  
	AddressV	= CLAMP;
};


struct VertexShaderInput
{
    float4 Position : POSITION0;
    float2 TexCoord : TEXCOORD0;
    
};

struct VertexShaderOutput
{
    float4 Position : POSITION0;
	float2 TexCoord : TEXCOORD0;
	float3 World    : TEXCOORD1; 
	
};

VertexShaderOutput VertexShaderFunction(VertexShaderInput input)
{
    VertexShaderOutput output;

    float4 worldPosition = mul(input.Position, World);
    float4 viewPosition = mul(worldPosition, View);
    output.Position = mul(viewPosition, Projection);   
    output.TexCoord= input.TexCoord;
    output.World=worldPosition.xyz;

    return output;
}

float4 PixelShaderFunction(VertexShaderOutput input) : COLOR0
{
	float4 noise=float4(0,0,0,0);
	
	// texture the pixel
	float2 tc = input.TexCoord;
	tc =(tc-0.5)/2;
	tc.x=length(tc)*4;
	tc.y=0;
    float4 result = tex2D(RingSampler,tc);
	result.w*=0.5;

	float3 depth = CameraPosition-input.World;
	float blend=length(depth);
	if (blend<100)
	{
		blend/=100;
		noise=(tex2D(NoiseSampler,input.TexCoord));
	}else{
		blend=1;
	}
	
    float3 dst = LightPosition;
    float3 Ray = LightPosition-input.World;
    float lr = length(Ray);
    float ld = length(dst);
    Ray=normalize(Ray);
	float B = dot(dst,Ray);
	float C = dot(dst, dst) - scale2;
	float D = B*B - C;
	if (D>0)
	{
		if (lr<ld)
		{
			result.xyz=0;
		}
		
	}
	result.xyz = lerp(noise.xyz,result.xyz,blend);
    return result;
}

technique Technique1
{
    pass Pass1
    {
        // TODO: set renderstates here.

        VertexShader = compile vs_1_1 VertexShaderFunction();
        PixelShader = compile ps_3_0 PixelShaderFunction();
    }
}
