using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;

namespace PlanetTutorial
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        public static int TextureWidth = 512;
        public static int TextureHeight = 512;
        public static float ESun = 1;
        public static float CLand = 0.9f;
        public static float CWater = 0.2f;
        public static int PlanetWeatherIntensity = 16;
        public static float AtmosphericDensity = 1.2f;
        public static float AtmosphericViscosity = 0.00018f;
        public static float PlanetSurfaceVelocity = 6371000.0f * 2.0f * MathHelper.Pi / 86400.0f;
        public static float VelocityScale = 0.1f;
        public static Texture2D mercator;
        public static Texture2D noise;
        public static Texture2D noiseBump;

        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;
        Planet planet;
        public static Model sphere;
        public static Model ring;

        Model SkySphere;
        Effect SkySphereEffect;

        Matrix Projection;
        Matrix View;
        
        public static Vector3 CameraPosition;
        public static Vector3 LightPosition;

        PlanetType ptype = PlanetType.Ice;
        PlanetType currenttype = PlanetType.Undefined;
        GamePadState oldgps;
        int Seed = 0;
        SpriteFont font;
        float angle;

        public static GenerateClouds cloudgen;

        Star star;
        
        

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = 1024;
            graphics.PreferredBackBufferHeight = 768;
            Content.RootDirectory = "Content";
            
            
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            CameraPosition = new Vector3(0, 0, -400);
            LightPosition = new Vector3(-2000, 0, 2000);

            Projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.Pi / 2.0f, 1024.0f / 768.0f, 1, 20000);

            cloudgen = new GenerateClouds();
            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);
            sphere = Content.Load<Model>("sphere2");
            font = Content.Load<SpriteFont>("SpriteFont1");
            ring = Content.Load<Model>("ring");
            noise = Content.Load<Texture2D>("Textures\\noise");

            oldgps = GamePad.GetState(PlayerIndex.One);
            cloudgen.Load(GraphicsDevice, Content);
            noiseBump = cloudgen.BumpMap(noise, 128);

            SkySphereEffect = Content.Load<Effect>("Shaders\\SkySphere");
            TextureCube SkyboxTexture = Content.Load<TextureCube>("Textures\\space");
            SkySphere = Content.Load<Model>("SphereHighPoly");
            SkySphereEffect.Parameters["SkyboxTexture"].SetValue(SkyboxTexture);
            // Set the Skysphere Effect to each part of the Skysphere model
            foreach (ModelMesh mesh in SkySphere.Meshes)
            {
                foreach (ModelMeshPart part in mesh.MeshParts)
                {
                    part.Effect = SkySphereEffect;
                }
            }

            star = new Star();
            star.SetType(Star.StarType.G);
            star.LoadContent(GraphicsDevice, Content);            
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            float dt = gameTime.ElapsedGameTime.Milliseconds/1000.0f;

            // move the light source
            angle += (float)gameTime.ElapsedGameTime.Milliseconds / 10000.0f;
            LightPosition.X = (float)(10000.0f * Math.Cos(angle));
            LightPosition.Z = (float)(10000.0f * Math.Sin(angle));

            
            // player input
            GamePadState gps = GamePad.GetState(PlayerIndex.One);
            if ((gps.Buttons.A == ButtonState.Pressed) && (oldgps.Buttons.A != ButtonState.Pressed))
            {
                star.type++;
                if (star.type == Star.StarType.Undefined)
                    star.type = Star.StarType.M;
                star.SetType(star.type);
            }

            if ((gps.DPad.Right == ButtonState.Pressed) && (oldgps.DPad.Right != ButtonState.Pressed))
            {
                ptype++;
                if (ptype > PlanetType.Molten)
                    ptype = PlanetType.Ice;

            }
            if ((gps.DPad.Left == ButtonState.Pressed) && (oldgps.DPad.Left != ButtonState.Pressed))
            {
                ptype--;
                if (ptype < PlanetType.Ice)
                    ptype = PlanetType.Molten;


            }
            if ((gps.DPad.Up == ButtonState.Pressed) && (oldgps.DPad.Up != ButtonState.Pressed))
            {
                Seed++;
                currenttype = PlanetType.Undefined;
            }
            if ((gps.DPad.Down == ButtonState.Pressed) && (oldgps.DPad.Down != ButtonState.Pressed))
            {
                Seed--;
                currenttype = PlanetType.Undefined;
            }
            CameraPosition.Z += gps.Triggers.Right * 100 * dt;
            CameraPosition.Z -= gps.Triggers.Left * 100 * dt;
            if (gps.ThumbSticks.Left.Y < -0.5)
                CameraPosition.Y -= 50 * dt;
            if (gps.ThumbSticks.Left.Y > 0.5)
                CameraPosition.Y += 50 * dt;

            View = Matrix.CreateLookAt(CameraPosition, Vector3.Zero, Vector3.Up);


            if (planet != null)
            {
                if (gps.ThumbSticks.Right.Y < -0.5)
                {
                    planet.xOvercast -= 0.01f;
                    planet.Generate(GraphicsDevice);
                }
                if (gps.ThumbSticks.Right.Y > 0.5)
                {
                    planet.xOvercast += 0.01f;
                    planet.Generate(GraphicsDevice);
                }
                
                if (gps.ThumbSticks.Left.X < -0.5)
                {
                    planet.roll -= 0.05f;
                }
                if (gps.ThumbSticks.Left.X > 0.5)
                {
                    planet.roll += 0.05f;
                }
            }
            oldgps = gps;
            if (currenttype != ptype)
            {
                switch (ptype)
                {
                    case PlanetType.Ice:
                        IcePlanet iplanet = new IcePlanet(GraphicsDevice,Seed);
                        iplanet.LoadContent(Content);
                        planet = iplanet;
                        break;

                    case PlanetType.Gas:
                        GasGiant gplanet = new GasGiant(GraphicsDevice, Seed);
                        gplanet.LoadContent(Content);
                        planet = gplanet;
                        break;

                    case PlanetType.Molten:
                        Molten mplanet = new Molten(GraphicsDevice, Seed);
                        mplanet.LoadContent(Content);
                        planet = mplanet;
                        mercator = mplanet.Mercator;
                        cloudgen.Generate();
                        break;

                    case PlanetType.Rock:
                        Rock rplanet = new Rock(GraphicsDevice, Seed);
                        rplanet.LoadContent(Content);
                        planet = rplanet;
                        break;

                    case PlanetType.Water:
                        WaterPlanet wplanet = new WaterPlanet(GraphicsDevice, Seed);
                        wplanet.LoadContent(Content);
                        planet = wplanet;
                        mercator = wplanet.Mercator;
                        cloudgen.Generate();
                        break;



                }
                planet.xOvercast = 1.1f;
                currenttype = ptype;
            }
            base.Update(gameTime);
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            // Set the View and Projection matrix for the effect
            SkySphereEffect.Parameters["ViewMatrix"].SetValue(View*Matrix.CreateRotationY(-angle));
            SkySphereEffect.Parameters["ProjectionMatrix"].SetValue(Projection);
            // Draw the sphere model that the effect projects onto
            foreach (ModelMesh mesh in SkySphere.Meshes)
            {
                mesh.Draw();
            }
            GraphicsDevice.Clear(ClearOptions.DepthBuffer,Color.Black,1000000,0);

            GraphicsDevice.RenderState.DepthBufferWriteEnable = true;
            GraphicsDevice.RenderState.DepthBufferEnable = true;
            GraphicsDevice.RenderState.CullMode = CullMode.None;
            planet.Draw(View, Projection);
            GraphicsDevice.RenderState.DepthBufferWriteEnable = true;
            GraphicsDevice.RenderState.DepthBufferEnable = true;
            GraphicsDevice.RenderState.CullMode = CullMode.None;
            star.Draw(View, Projection);
            if (planet != null)
            {
                spriteBatch.Begin();
                spriteBatch.DrawString(font, string.Format("xOvercast    {0}", planet.xOvercast), new Vector2(10, 10), Color.White);
                spriteBatch.DrawString(font, string.Format("Seed         {0}", Seed), new Vector2(10, 30), Color.White);
                spriteBatch.DrawString(font, string.Format("Type         {0}", planet.Type), new Vector2(10, 50), Color.White);
                spriteBatch.DrawString(font, string.Format("SubType      {0}", planet.SubType), new Vector2(10, 70), Color.White);
                spriteBatch.DrawString(font, string.Format("Star class   {0}", star.type), new Vector2(10, 90), Color.White);                spriteBatch.End();
            }

           
            base.Draw(gameTime);
            

        }
    }
}
